@@ -1,15 +1,16 @@
 """All internal ansible-lint rules."""
+from __future__ import annotations
+
 import copy
-import glob
-import importlib.util
 import inspect
 import logging
-import os
 import re
+import sys
 from argparse import Namespace
 from collections import defaultdict
 from functools import lru_cache
-from importlib.abc import Loader
+from importlib import import_module
+from pathlib import Path
 from typing import (
     Any,
     Dict,
@@ -36,6 +37,7 @@ from ansiblelint._internal.rules import (
 )
 from ansiblelint.config import PROFILES, get_rule_config
 from ansiblelint.config import options as default_options
+from ansiblelint.constants import RULE_DOC_URL
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, expand_paths_vars
 
@@ -56,7 +58,12 @@ class AnsibleLintRule(BaseRule):
     """AnsibleLintRule should be used as base for writing new rules."""
 
     @property
-    def rule_config(self) -> Dict[str, Any]:
+    def url(self) -> str:
+        """Return rule documentation url."""
+        return RULE_DOC_URL + self.id + "/"
+
+    @property
+    def rule_config(self) -> dict[str, Any]:
         """Retrieve rule specific configuration."""
         return get_rule_config(self.id)
 
@@ -76,10 +83,10 @@ class AnsibleLintRule(BaseRule):
     # pylint: disable=too-many-arguments
     def create_matcherror(
         self,
-        message: Optional[str] = None,
+        message: str | None = None,
         linenumber: int = 1,
         details: str = "",
-        filename: Optional[Union[str, Lintable]] = None,
+        filename: Lintable | None = None,
         tag: str = "",
     ) -> MatchError:
         """Instantiate a new MatchError."""
@@ -94,7 +101,7 @@ class AnsibleLintRule(BaseRule):
             match.tag = tag
         # search through callers to find one of the match* methods
         frame = inspect.currentframe()
-        match_type: Optional[str] = None
+        match_type: str | None = None
         while not match_type and frame is not None:
             func_name = frame.f_code.co_name
             match_type = match_types.get(func_name, None)
@@ -107,7 +114,7 @@ class AnsibleLintRule(BaseRule):
 
     @staticmethod
     def _enrich_matcherror_with_task_details(
-        match: MatchError, task: Dict[str, Any]
+        match: MatchError, task: dict[str, Any]
     ) -> None:
         match.task = task
         if not match.details:
@@ -115,8 +122,8 @@ class AnsibleLintRule(BaseRule):
         if match.linenumber < task[ansiblelint.utils.LINE_NUMBER_KEY]:
             match.linenumber = task[ansiblelint.utils.LINE_NUMBER_KEY]
 
-    def matchlines(self, file: Lintable) -> List[MatchError]:
-        matches: List[MatchError] = []
+    def matchlines(self, file: Lintable) -> list[MatchError]:
+        matches: list[MatchError] = []
         if not self.match:
             return matches
         # arrays are 0-based, line numbers are 1-based
@@ -144,8 +151,8 @@ class AnsibleLintRule(BaseRule):
             matches.append(matcherror)
         return matches
 
-    def matchtasks(self, file: Lintable) -> List[MatchError]:
-        matches: List[MatchError] = []
+    def matchtasks(self, file: Lintable) -> list[MatchError]:
+        matches: list[MatchError] = []
         if (
             not self.matchtask
             or file.kind not in ["handlers", "tasks", "playbook"]
@@ -188,8 +195,8 @@ class AnsibleLintRule(BaseRule):
             matches.append(match)
         return matches
 
-    def matchyaml(self, file: Lintable) -> List[MatchError]:
-        matches: List[MatchError] = []
+    def matchyaml(self, file: Lintable) -> list[MatchError]:
+        matches: list[MatchError] = []
         if not self.matchplay or str(file.base_kind) != "text/yaml":
             return matches
 
@@ -200,7 +207,7 @@ class AnsibleLintRule(BaseRule):
         if isinstance(yaml, str):
             if yaml.startswith("$ANSIBLE_VAULT"):
                 return []
-            return [MatchError(filename=str(file.path), rule=LoadingFailureRule())]
+            return [MatchError(filename=file, rule=LoadingFailureRule())]
         if not yaml:
             return matches
 
@@ -234,7 +241,7 @@ class TransformMixin:
         self,
         match: MatchError,
         lintable: Lintable,
-        data: Union[CommentedMap, CommentedSeq, str],
+        data: CommentedMap | CommentedSeq | str,
     ) -> None:
         """Transform ``data`` to try to fix the MatchError identified by this rule.
 
@@ -270,8 +277,8 @@ class TransformMixin:
 
     @staticmethod
     def seek(
-        yaml_path: List[Union[int, str]],
-        data: Union[MutableMapping[str, Any], MutableSequence[Any], str],
+        yaml_path: list[int | str],
+        data: MutableMapping[str, Any] | MutableSequence[Any] | str,
     ) -> Any:
         """Get the element identified by ``yaml_path`` in ``data``.
 
@@ -299,43 +306,48 @@ class TransformMixin:
         return target
 
 
-def is_valid_rule(rule: Any) -> bool:
-    """Check if given rule is valid or not."""
-    return issubclass(rule, AnsibleLintRule) and bool(rule.id) and bool(rule.shortdesc)
-
-
 # pylint: disable=too-many-nested-blocks
-def load_plugins(directory: str) -> Iterator[AnsibleLintRule]:
+def load_plugins(  # noqa: max-complexity: 11
+    dirs: list[str],
+) -> Iterator[AnsibleLintRule]:
     """Yield a rule class."""
-    for pluginfile in glob.glob(os.path.join(directory, "[A-Za-z]*.py")):
 
-        pluginname = os.path.basename(pluginfile.replace(".py", ""))
-        spec = importlib.util.spec_from_file_location(pluginname, pluginfile)
+    def all_subclasses(cls: type) -> set[type]:
+        return set(cls.__subclasses__()).union(
+            [s for c in cls.__subclasses__() for s in all_subclasses(c)]
+        )
 
-        # https://github.com/python/typeshed/issues/2793
-        if spec and isinstance(spec.loader, Loader):
+    orig_sys_path = sys.path.copy()
 
-            # load rule markdown documentation if found
-            help_md = ""
-            if spec.origin:
-                md_filename = os.path.splitext(spec.origin)[0] + ".md"
-                if os.path.exists(md_filename):
-                    with open(md_filename, encoding="utf-8") as f:
-                        help_md = f.read()
+    for directory in dirs:
+        if directory not in sys.path:
+            sys.path.append(str(directory))
 
-            module = importlib.util.module_from_spec(spec)
-            spec.loader.exec_module(module)
-            try:
-                for _, obj in inspect.getmembers(module):
-                    if inspect.isclass(obj) and is_valid_rule(obj):
-                        if help_md:
-                            obj.help = help_md
-                        yield obj()
+        # load all modules in the directory
+        for f in Path(directory).glob("*.py"):
+            if "__" not in f.stem and f.stem not in "conftest":
+                try:
+                    import_module(f"{f.stem}")
 
-            except (TypeError, ValueError, AttributeError) as exc:
-                _logger.warning(
-                    "Skipped invalid rule from %s due to %s", pluginname, exc
-                )
+                except ImportError as exc:
+                    _logger.warning("Ignore loading rule from %s due to %s", f, exc)
+    # restore sys.path
+    sys.path = orig_sys_path
+
+    rules: dict[str, BaseRule] = {}
+    for rule in all_subclasses(BaseRule):
+        # we do not return the rules that are not loaded from passed 'directory'
+        # or rules that do not have a valid id. For example, during testing
+        # python may load other rule classes, some outside the tested rule
+        # directories.
+        if getattr(rule, "id") and Path(inspect.getfile(rule)).parent.absolute() in [
+            Path(x).absolute() for x in dirs
+        ]:
+            if issubclass(rule, BaseRule) and rule.id not in rules:
+                rules[rule.id] = rule()
+    for rule in rules.values():  # type: ignore
+        if isinstance(rule, AnsibleLintRule) and bool(rule.id):
+            yield rule
 
 
 class RulesCollection:
@@ -343,7 +355,7 @@ class RulesCollection:
 
     def __init__(
         self,
-        rulesdirs: Optional[List[str]] = None,
+        rulesdirs: list[str] | None = None,
         options: Namespace = default_options,
     ) -> None:
         """Initialize a RulesCollection instance."""
@@ -351,16 +363,14 @@ class RulesCollection:
         if rulesdirs is None:
             rulesdirs = []
         self.rulesdirs = expand_paths_vars(rulesdirs)
-        self.rules: List[BaseRule] = []
+        self.rules: list[BaseRule] = []
         # internal rules included in order to expose them for docs as they are
         # not directly loaded by our rule loader.
         self.rules.extend(
             [RuntimeErrorRule(), AnsibleParserErrorRule(), LoadingFailureRule()]
         )
-        for rulesdir in self.rulesdirs:
-            _logger.debug("Loading rules from %s", rulesdir)
-            for rule in load_plugins(rulesdir):
-                self.register(rule)
+        for rule in load_plugins(rulesdirs):
+            self.register(rule)
         self.rules = sorted(self.rules)
 
     def register(self, obj: AnsibleLintRule) -> None:
@@ -389,18 +399,18 @@ class RulesCollection:
         """Return the length of the RulesCollection data."""
         return len(self.rules)
 
-    def extend(self, more: List[AnsibleLintRule]) -> None:
+    def extend(self, more: list[AnsibleLintRule]) -> None:
         """Combine rules."""
         self.rules.extend(more)
 
     def run(  # noqa: max-complexity: 12
         self,
         file: Lintable,
-        tags: Optional[Set[str]] = None,
-        skip_list: Optional[List[str]] = None,
-    ) -> List[MatchError]:
+        tags: set[str] | None = None,
+        skip_list: list[str] | None = None,
+    ) -> list[MatchError]:
         """Run all the rules against the given lintable."""
-        matches: List[MatchError] = []
+        matches: list[MatchError] = []
         if tags is None:
             tags = set()
         if skip_list is None:
@@ -410,7 +420,7 @@ class RulesCollection:
             try:
                 if file.content is not None:  # loads the file content
                     pass
-            except (IOError, UnicodeDecodeError) as exc:
+            except (OSError, UnicodeDecodeError) as exc:
                 return [
                     MatchError(
                         message=str(exc),
